@@ -4,16 +4,16 @@
  * Copyright (C) 2008, Nokia
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA  02110-1301, USA.
@@ -49,7 +49,7 @@ row_add (GPtrArray *row,
 	g_ptr_array_add (row, elem);
 }
 
-static inline void
+static inline gboolean
 row_insert (GPtrArray *row, 
 	    gchar     *value, 
 	    guint      lindex)
@@ -67,12 +67,14 @@ row_insert (GPtrArray *row,
 	 */
 	for (iter = list; iter; iter=iter->next) {
 		if (strcmp (iter->data, value) == 0) {
-			return;
+			return FALSE;
 		}
 	}
 
 	list = g_slist_prepend (list, value);
 	elem->value = list;
+
+	return TRUE;
 }
 
 static inline void
@@ -124,6 +126,7 @@ rows_destroy (GPtrArray *rows)
 
 	for (i = 0; i < rows->len; i++) {
 		OneRow *row;
+
 		row = g_ptr_array_index (rows, i);
 		row_destroy (row->value);
 		g_slice_free (OneRow, row);
@@ -268,16 +271,19 @@ tracker_dbus_query_result_columns_to_strv (TrackerDBResultSet *result_set,
 {
 	gchar    **strv = NULL;
 	gint	   i = 0;
-	gint	   columns;
+	gint	   columns = 0;
 	gint       row_counter = 0;
 	gboolean   valid = TRUE;
 
-	if (result_set) {
-		columns = tracker_db_result_set_get_n_columns (result_set);
-		if (rewind) {
-			 /* Make sure we rewind before iterating the result set */
-			tracker_db_result_set_rewind (result_set);
-		}
+	if (!result_set) {
+		return NULL;
+	}
+
+	columns = tracker_db_result_set_get_n_columns (result_set);
+
+	if (rewind) {
+		/* Make sure we rewind before iterating the result set */
+		tracker_db_result_set_rewind (result_set);
 	}
 
 	if (!result_set || offset_column > columns) {
@@ -303,7 +309,7 @@ tracker_dbus_query_result_columns_to_strv (TrackerDBResultSet *result_set,
 			g_value_init (&transform, G_TYPE_STRING);
 			
 			_tracker_db_result_set_get_value (result_set, i, &value);
-			if (g_value_transform (&value, &transform)) {
+			if (G_IS_VALUE (&value) && g_value_transform (&value, &transform)) {
 				if (row_counter == 0) {
 					strv[i] = g_value_dup_string (&transform);
 				} else {
@@ -323,8 +329,10 @@ tracker_dbus_query_result_columns_to_strv (TrackerDBResultSet *result_set,
 					}
 					
 				}
+				g_value_unset (&value);
+			} else if (row_counter == 0) {
+				strv[i] = g_strdup ("");
 			}
-			g_value_unset (&value);
 			g_value_unset (&transform);
 		}
 
@@ -360,22 +368,21 @@ tracker_dbus_query_result_to_hash_table (TrackerDBResultSet *result_set)
 	}
 
 	while (valid) {
-		GValue	 transform = { 0, };
 		GValue	*values;
 		gchar  **p;
 		gint	 i = 0;
 		gchar	*key;
 		GSList	*list = NULL;
 
-		g_value_init (&transform, G_TYPE_STRING);
-
 		tracker_db_result_set_get (result_set, 0, &key, -1);
 		values = tracker_dbus_gvalue_slice_new (G_TYPE_STRV);
 
 		for (i = 1; i < field_count; i++) {
+			GValue  transform = { 0, };
 			GValue	value = { 0, };
 			gchar  *str;
 
+			g_value_init (&transform, G_TYPE_STRING);
 			_tracker_db_result_set_get_value (result_set, i, &value);
 
 			if (g_value_transform (&value, &transform)) {
@@ -386,16 +393,23 @@ tracker_dbus_query_result_to_hash_table (TrackerDBResultSet *result_set)
 					g_free (str);
 					str = g_strdup ("");
 				}
+
+				g_value_unset (&transform);
 			} else {
 				str = g_strdup ("");
 			}
 
 			list = g_slist_prepend (list, (gchar*) str);
+
+			g_value_unset (&value);
 		}
 
 		list = g_slist_reverse (list);
 		p = tracker_dbus_slist_to_strv (list);
+
+		g_slist_foreach (list, (GFunc)g_free, NULL);
 		g_slist_free (list);
+
 		g_value_take_boxed (values, p);
 		g_hash_table_insert (hash_table, key, values);
 
@@ -433,29 +447,25 @@ tracker_dbus_query_result_to_ptr_array (TrackerDBResultSet *result_set)
 		for (i = 0; i < columns; i++) {
 			GValue	transform = { 0, };
 			GValue	value = { 0, };
-			gchar  *str;
+			gchar  *str = NULL;
 
 			g_value_init (&transform, G_TYPE_STRING);
 
 			_tracker_db_result_set_get_value (result_set, i, &value);
 
-			if (g_value_transform (&value, &transform)) {
+			if (G_IS_VALUE (&value) && g_value_transform (&value, &transform)) {
 				str = g_value_dup_string (&transform);
+			}
 
-				if (!str) {
-					str = g_strdup ("");
-				} else if (!g_utf8_validate (str, -1, NULL)) {
-					g_warning ("Could not add string:'%s' to GStrv, invalid UTF-8", str);
-					g_free (str);
-					str = g_strdup ("");
-				}
-			} else {
+			if (!str) {
 				str = g_strdup ("");
 			}
 
 			list = g_slist_prepend (list, (gchar*) str);
 
-			g_value_unset (&value);
+			if (G_IS_VALUE (&value)) {
+				g_value_unset (&value);
+			}
 			g_value_unset (&transform);
 		}
 
@@ -538,14 +548,16 @@ tracker_dbus_query_result_multi_to_ptr_array (TrackerDBResultSet *result_set)
 			if (add) {
 				row_add (row, str);
 			} else {				
-				row_insert (row, str, column-1);
+				if (!row_insert (row, str, column-1)) {
+					/* Failed to insert */
+					g_free (str);
+				}
 			}		       
 
 			g_value_unset (&value);
 			g_value_unset (&transform);
 		}
 
-
 		if (add) {
 			rows_add (rows, key, row);
 		}
@@ -553,7 +565,7 @@ tracker_dbus_query_result_multi_to_ptr_array (TrackerDBResultSet *result_set)
 		valid = tracker_db_result_set_iter_next (result_set);
 	}
 
-	result = g_ptr_array_new();
+	result = g_ptr_array_new ();
 
 	rows_migrate (rows, result);
 	rows_destroy (rows);
