@@ -30,6 +30,16 @@
 #include "tracker-miner-client.h"
 #include "tracker-miner-dbus.h"
 
+/**
+ * SECTION:tracker-miner-manager
+ * @short_description: External control and monitoring of miners
+ * @include: libtracker-miner/tracker-miner-manager.h
+ *
+ * #TrackerMinerManager keeps track of available miners, their current
+ * progress/status, and also allows basic external control on them, such
+ * as pausing or resuming data processing.
+ **/
+
 #define TRACKER_MINER_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TRACKER_TYPE_MINER_MANAGER, TrackerMinerManagerPrivate))
 
 #define DESKTOP_ENTRY_GROUP "Desktop Entry"
@@ -80,6 +90,16 @@ tracker_miner_manager_class_init (TrackerMinerManagerClass *klass)
 
 	object_class->finalize = miner_manager_finalize;
 
+	/**
+	 * TrackerMinerManager::miner-progress
+	 * @manager: the #TrackerMinerManager
+	 * @miner: miner reference
+	 * @status: miner status
+	 * @progress: miner progress, from 0 to 1
+	 *
+	 * The ::miner-progress signal is meant to report status/progress changes
+	 * in any tracked miner.
+	 **/
 	signals [MINER_PROGRESS] =
 		g_signal_new ("miner-progress",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -91,6 +111,14 @@ tracker_miner_manager_class_init (TrackerMinerManagerClass *klass)
 			      G_TYPE_STRING,
 			      G_TYPE_STRING,
 			      G_TYPE_DOUBLE);
+	/**
+	 * TrackerMinerManager::miner-paused
+	 * @manager: the #TrackerMinerManager
+	 * @miner: miner reference
+	 *
+	 * The ::miner-paused signal will be emitted whenever a miner
+	 * (referenced by @miner) is paused.
+	 **/
 	signals [MINER_PAUSED] =
 		g_signal_new ("miner-paused",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -100,6 +128,14 @@ tracker_miner_manager_class_init (TrackerMinerManagerClass *klass)
 			      g_cclosure_marshal_VOID__STRING,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_STRING);
+	/**
+	 * TrackerMinerManager::miner-resumed
+	 * @manager: the #TrackerMinerManager
+	 * @miner: miner reference
+	 *
+	 * The ::miner-resumed signal will be emitted whenever a miner
+	 * (referenced by @miner) is resumed.
+	 **/
 	signals [MINER_RESUMED] =
 		g_signal_new ("miner-resumed",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -109,6 +145,15 @@ tracker_miner_manager_class_init (TrackerMinerManagerClass *klass)
 			      g_cclosure_marshal_VOID__STRING,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_STRING);
+	/**
+	 * TrackerMinerManager::miner-activated
+	 * @manager: the #TrackerMinerManager
+	 * @miner: miner reference
+	 *
+	 * The ::miner-activated signal will be emitted whenever a miner
+	 * (referenced by @miner) is activated (technically, this means
+	 * the miner has appeared in the session bus).
+	 **/
 	signals [MINER_ACTIVATED] =
 		g_signal_new ("miner-activated",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -118,6 +163,15 @@ tracker_miner_manager_class_init (TrackerMinerManagerClass *klass)
 			      g_cclosure_marshal_VOID__STRING,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_STRING);
+	/**
+	 * TrackerMinerManager::miner-deactivated
+	 * @manager: the #TrackerMinerManager
+	 * @miner: miner reference
+	 *
+	 * The ::miner-deactivated signal will be emitted whenever a miner
+	 * (referenced by @miner) is deactivated (technically, this means
+	 * the miner has disappeared from the session bus).
+	 **/
 	signals [MINER_DEACTIVATED] =
 		g_signal_new ("miner-deactivated",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -690,8 +744,14 @@ tracker_miner_manager_get_status (TrackerMinerManager  *manager,
 	org_freedesktop_Tracker1_Miner_get_progress (proxy, &p, &error);
 
 	if (error) {
-		g_critical ("Could not get miner progress for '%s': %s", miner,
-			    error->message);
+		/* We handle this error as a special case, some
+		 * plugins don't have .service files. 
+		 */
+		if (error->code != DBUS_GERROR_SERVICE_UNKNOWN) {
+			g_critical ("Could not get miner progress for '%s': %s", miner,
+				    error->message);
+		}
+
 		g_error_free (error);
 
 		return FALSE;
@@ -854,3 +914,55 @@ tracker_miner_manager_get_description (TrackerMinerManager *manager,
 
 	return NULL;
 }
+
+
+/**
+ * tracker_miner_manager_writeback:
+ * @manager: a #TrackerMinerManager.
+ * @miner: miner reference
+ * @subjects: subjects to mark as writeback
+ *
+ * Asks @miner to mark @subjects as writeback
+ *
+ * Returns: %TRUE if the miner was asked to writeback successfully.
+ **/
+gboolean
+tracker_miner_manager_writeback (TrackerMinerManager *manager,
+                                 const gchar         *miner,
+                                 const gchar        **subjects)
+{
+	DBusGProxy *proxy;
+	const gchar *app_name;
+	GError *error = NULL;
+
+	g_return_val_if_fail (TRACKER_IS_MINER_MANAGER (manager), FALSE);
+	g_return_val_if_fail (miner != NULL, FALSE);
+
+	proxy = find_miner_proxy (manager, miner);
+
+	if (!proxy) {
+		g_warning ("No D-Bus proxy found for miner '%s'", miner);
+		return FALSE;
+	}
+
+	/* Find a reasonable app name */
+	app_name = g_get_application_name ();
+
+	if (!app_name) {
+		app_name = g_get_prgname ();
+	}
+
+	if (!app_name) {
+		app_name = "TrackerMinerManager client";
+	}
+
+	org_freedesktop_Tracker1_Miner_writeback (proxy, subjects, &error);
+
+	if (error) {
+		g_warning ("Could not writeback miner '%s': %s", miner, error->message);
+		g_error_free (error);
+		return FALSE;
+	}
+
+	return TRUE;
+}
