@@ -295,7 +295,7 @@ extract_jpeg (const gchar          *uri,
 			g_free (ed.make);
 		}
 
-		md.title = tracker_coalesce (2, xd.title, ed.document_name);
+		md.title = tracker_coalesce (2, xd.title, xd.Title, ed.document_name);
 		md.orientation = tracker_coalesce (3, ed.orientation, xd.Orientation, id.image_orientation);
 		md.copyright = tracker_coalesce (3, ed.copyright, xd.rights, id.copyright_notice);
 		md.white_balance = tracker_coalesce (2, ed.white_balance, xd.WhiteBalance);
