@@ -19,12 +19,18 @@
  */
 
 #include "config.h"
-#include "tracker-status-icon.h"
-#include <libtracker-miner/tracker-miner-manager.h>
+
 #include <string.h>
 #include <locale.h>
+
 #include <glib/gi18n.h>
 
+#include <libtracker-miner/tracker-miner-manager.h>
+
+#include "tracker-status-icon.h"
+#include "tracker-icon-config.h"
+#include "tomboykeybinder.h"
+
 #define TRACKER_STATUS_ICON_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TRACKER_TYPE_STATUS_ICON, TrackerStatusIconPrivate))
 
 typedef struct TrackerStatusIconPrivate TrackerStatusIconPrivate;
@@ -48,6 +54,7 @@ typedef enum {
 struct TrackerStatusIconPrivate {
 	GdkPixbuf *icons [ICON_LAST];
 	TrackerStatus current_status;
+	TrackerVisibility current_visibility;
 	guint animation_id;
 
 	TrackerMinerManager *manager;
@@ -56,6 +63,8 @@ struct TrackerStatusIconPrivate {
 	GtkSizeGroup *size_group;
 
 	GHashTable *miners;
+
+	TrackerIconConfig *config;
 };
 
 struct MinerMenuEntry {
@@ -64,43 +73,48 @@ struct MinerMenuEntry {
 	GtkWidget *state;
 	GtkWidget *name;
 	GtkWidget *progress_bar;
+	GtkWidget *progress_percentage;
 
+	gchar *status;
 	gdouble progress;
+	gboolean paused;
+	guint pulse_id;
 	guint32 cookie;
 	guint active : 1;
 };
 
-static void status_icon_constructed (GObject          *object);
-static void status_icon_finalize    (GObject          *object);
-
-static void status_icon_activate   (GtkStatusIcon     *icon);
-static void status_icon_popup_menu (GtkStatusIcon     *icon,
-				    guint              button,
-				    guint32            activate_time);
-
-static void status_icon_miner_progress (TrackerMinerManager *manager,
-					const gchar         *miner_name,
-					const gchar         *status,
-					gdouble              progress,
-					gpointer             user_data);
-static void status_icon_miner_paused   (TrackerMinerManager *manager,
-					const gchar         *miner_name,
-					gpointer             user_data);
-static void status_icon_miner_resumed  (TrackerMinerManager *manager,
-					const gchar         *miner_name,
-					gpointer             user_data);
-static void status_icon_miner_activated   (TrackerMinerManager *manager,
-					   const gchar         *miner_name,
-					   gpointer             user_data);
-static void status_icon_miner_deactivated (TrackerMinerManager *manager,
-					   const gchar         *miner_name,
-					   gpointer             user_data);
-
-static void        status_icon_initialize_miners_menu (TrackerStatusIcon *icon);
-static GtkWidget * status_icon_create_context_menu    (TrackerStatusIcon *icon);
-
-static void status_icon_set_status (TrackerStatusIcon *icon,
-				    TrackerStatus      status);
+static void       status_icon_constructed            (GObject             *object);
+static void       status_icon_finalize               (GObject             *object);
+static void       status_icon_activate               (GtkStatusIcon       *icon);
+static void       status_icon_popup_menu             (GtkStatusIcon       *icon,
+						      guint                button,
+						      guint32              activate_time);
+static void       status_icon_miner_progress         (TrackerMinerManager *manager,
+						      const gchar         *miner_name,
+						      const gchar         *status,
+						      gdouble              progress,
+						      gpointer             user_data);
+static void       status_icon_miner_paused           (TrackerMinerManager *manager,
+						      const gchar         *miner_name,
+						      gpointer             user_data);
+static void       status_icon_miner_resumed          (TrackerMinerManager *manager,
+						      const gchar         *miner_name,
+						      gpointer             user_data);
+static void       status_icon_miner_activated        (TrackerMinerManager *manager,
+						      const gchar         *miner_name,
+						      gpointer             user_data);
+static void       status_icon_miner_deactivated      (TrackerMinerManager *manager,
+						      const gchar         *miner_name,
+						      gpointer             user_data);
+static void       status_icon_visibility_notify      (TrackerIconConfig   *config,
+						      GParamSpec          *pspec,
+						      gpointer             user_data);
+static void       status_icon_initialize_miners_menu (TrackerStatusIcon   *icon);
+static GtkWidget *status_icon_create_context_menu    (TrackerStatusIcon   *icon);
+static void       status_icon_set_status             (TrackerStatusIcon   *icon,
+						      TrackerStatus        status);
+static void       launch_application_on_screen       (GdkScreen           *screen,
+						      const gchar         *command_line);
 
 G_DEFINE_TYPE (TrackerStatusIcon, tracker_status_icon, GTK_TYPE_STATUS_ICON)
 
@@ -119,6 +133,38 @@ tracker_status_icon_class_init (TrackerStatusIconClass *klass)
 	g_type_class_add_private (klass, sizeof (TrackerStatusIconPrivate));
 }
 
+static void
+miner_menu_entry_free (MinerMenuEntry *entry)
+{
+	if (entry->pulse_id) {
+		g_source_remove (entry->pulse_id);
+		entry->pulse_id = 0;
+	}
+
+	g_free (entry->status);
+	g_free (entry);
+}
+
+static void
+keybinding_activated_cb (gchar    *keybinding,
+			 gpointer  user_data)
+{
+	TrackerStatusIcon *icon = user_data;
+	GdkScreen *screen;
+
+	screen = gtk_status_icon_get_screen (GTK_STATUS_ICON (icon));
+	launch_application_on_screen (screen, "tracker-search-tool");
+}
+
+static void
+set_global_keybinding (TrackerStatusIcon *icon,
+		       const gchar       *keybinding)
+{
+	tomboy_keybinder_bind (keybinding,
+			       keybinding_activated_cb,
+			       icon);
+}
+
 static void
 tracker_status_icon_init (TrackerStatusIcon *icon)
 {
@@ -150,7 +196,7 @@ tracker_status_icon_init (TrackerStatusIcon *icon)
 
 	priv->miners = g_hash_table_new_full (g_str_hash, g_str_equal,
 					      (GDestroyNotify) g_free,
-					      (GDestroyNotify) g_free);
+					      (GDestroyNotify) miner_menu_entry_free);
 
 	priv->miner_menu = gtk_menu_new ();
 	priv->context_menu = status_icon_create_context_menu (icon);
@@ -168,6 +214,13 @@ tracker_status_icon_init (TrackerStatusIcon *icon)
 	g_signal_connect (priv->manager, "miner-deactivated",
 			  G_CALLBACK (status_icon_miner_deactivated), icon);
 	status_icon_initialize_miners_menu (icon);
+
+	priv->config = tracker_icon_config_new ();
+	g_signal_connect (priv->config, "notify::visibility",
+			  G_CALLBACK (status_icon_visibility_notify), icon);
+
+	/* FIXME: Make this configurable */
+	set_global_keybinding (icon, "<Ctrl><Alt>S");
 }
 
 static void
@@ -202,6 +255,7 @@ status_icon_finalize (GObject *object)
 
 	g_object_unref (priv->manager);
 	g_object_unref (priv->size_group);
+	g_object_unref (priv->config);
 
 	G_OBJECT_CLASS (tracker_status_icon_parent_class)->finalize (object);
 }
@@ -279,9 +333,61 @@ update_icon_status (TrackerStatusIcon *icon)
 		status = STATUS_IDLE;
 	}
 
+	/* entry = g_hash_table_lookup (priv->miners, miner_name); */
+
 	status_icon_set_status (icon, status);
 }
 
+static gboolean
+status_icon_miner_pulse (gpointer user_data)
+{
+	MinerMenuEntry *entry = user_data;
+
+	gtk_progress_bar_pulse (GTK_PROGRESS_BAR (entry->progress_bar));
+
+	return TRUE;
+}
+
+static void
+status_icon_miner_pulse_start (MinerMenuEntry *entry)
+{
+	if (entry->pulse_id == 0) {
+		entry->pulse_id = g_timeout_add (100, status_icon_miner_pulse, entry);
+	}
+}
+
+static void
+status_icon_miner_pulse_stop (MinerMenuEntry *entry)
+{
+	if (entry->pulse_id != 0) {
+		g_source_remove (entry->pulse_id);
+		entry->pulse_id = 0;
+	}
+}
+
+static void
+status_icon_miner_progress_set (MinerMenuEntry *entry)
+{
+	gchar *progress_str;
+
+	if (!entry->paused) {
+		if (entry->progress == 0.00) {
+			status_icon_miner_pulse_start (entry);
+		} else {
+			status_icon_miner_pulse_stop (entry);
+			gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (entry->progress_bar), entry->progress);
+		}
+	} else {
+		status_icon_miner_pulse_stop (entry);
+	}
+
+	progress_str = g_strdup_printf ("%3.0f%%", entry->progress * 100);
+	gtk_label_set_text (GTK_LABEL (entry->progress_percentage), progress_str);
+	g_free (progress_str);
+	
+	gtk_widget_set_tooltip_text (entry->box, entry->status);
+}
+
 static void
 status_icon_miner_progress (TrackerMinerManager *manager,
 			    const gchar         *miner_name,
@@ -292,7 +398,6 @@ status_icon_miner_progress (TrackerMinerManager *manager,
 	TrackerStatusIconPrivate *priv;
 	TrackerStatusIcon *icon;
 	MinerMenuEntry *entry;
-	gchar *progress_str;
 
 	icon = TRACKER_STATUS_ICON (user_data);
 	priv = TRACKER_STATUS_ICON_GET_PRIVATE (icon);
@@ -303,15 +408,11 @@ status_icon_miner_progress (TrackerMinerManager *manager,
 		return;
 	}
 
-	gtk_widget_set_tooltip_text (entry->box, status);
-	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (entry->progress_bar), progress);
-
-	progress_str = g_strdup_printf ("%3.0f%%", progress * 100);
-	gtk_progress_bar_set_text (GTK_PROGRESS_BAR (entry->progress_bar), progress_str);
-	g_free (progress_str);
-
+	g_free (entry->status);
+	entry->status = g_strdup (status);
 	entry->progress = progress;
 
+	status_icon_miner_progress_set (entry);
 	update_icon_status (icon);
 }
 
@@ -333,6 +434,9 @@ status_icon_miner_paused (TrackerMinerManager *manager,
 		return;
 	}
 
+	entry->paused = TRUE;
+	status_icon_miner_progress_set (entry);
+
 	gtk_image_set_from_stock (GTK_IMAGE (entry->state),
 				  GTK_STOCK_MEDIA_PAUSE,
 				  GTK_ICON_SIZE_MENU);
@@ -358,6 +462,9 @@ status_icon_miner_resumed (TrackerMinerManager *manager,
 		return;
 	}
 
+	entry->paused = FALSE;
+	status_icon_miner_progress_set (entry);
+
 	gtk_image_set_from_stock (GTK_IMAGE (entry->state),
 				  GTK_STOCK_MEDIA_PLAY,
 				  GTK_ICON_SIZE_MENU);
@@ -384,6 +491,8 @@ status_icon_miner_activated (TrackerMinerManager *manager,
 	}
 
 	gtk_widget_set_sensitive (entry->menu_item, TRUE);
+	gtk_widget_show (entry->progress_bar);
+	gtk_widget_show (entry->progress_percentage);
 	entry->active = TRUE;
 
 	update_icon_status (icon);
@@ -408,6 +517,9 @@ status_icon_miner_deactivated (TrackerMinerManager *manager,
 	}
 
 	gtk_widget_set_sensitive (entry->menu_item, FALSE);
+	gtk_widget_hide (entry->progress_bar);
+	gtk_widget_hide (entry->progress_percentage);
+
 	status_icon_miner_progress (priv->manager, miner_name,
 				    _("Miner is not running"), 0.0, icon);
 	entry->active = FALSE;
@@ -418,6 +530,16 @@ status_icon_miner_deactivated (TrackerMinerManager *manager,
 	update_icon_status (icon);
 }
 
+static void
+status_icon_visibility_notify (TrackerIconConfig *config,
+			       GParamSpec        *pspec,
+			       gpointer           user_data)
+{
+	TrackerStatusIcon *icon = user_data;
+
+	update_icon_status (icon);
+}
+
 static void
 miner_menu_entry_activate_cb (GtkMenuItem *item,
 			      gpointer     user_data)
@@ -457,24 +579,48 @@ miner_menu_entry_add (TrackerStatusIcon *icon,
 {
 	TrackerStatusIconPrivate *priv;
 	MinerMenuEntry *entry;
+        PangoFontDescription *fontdesc;
+        PangoFontMetrics *metrics;
+        PangoContext *context;
+        PangoLanguage *lang;
 	const gchar *name;
 	gchar *str;
+        gint ascent;
 
 	priv = TRACKER_STATUS_ICON_GET_PRIVATE (icon);
 	name = tracker_miner_manager_get_display_name (priv->manager, miner);
 	str = g_strdup (miner);
 
 	entry = g_new0 (MinerMenuEntry, 1);
+
 	entry->box = gtk_hbox_new (FALSE, 6);
 	entry->state = gtk_image_new_from_stock (GTK_STOCK_MEDIA_PLAY,
 						 GTK_ICON_SIZE_MENU);
 	entry->name = gtk_label_new (name);
-	gtk_misc_set_alignment (GTK_MISC (entry->name), 0, 0.5);
+	gtk_misc_set_alignment (GTK_MISC (entry->name), 0.0, 0.5);
+	gtk_box_pack_start (GTK_BOX (entry->box), entry->name, TRUE, TRUE, 0);
+
+	entry->progress_percentage = gtk_label_new ("100%");
+	gtk_misc_set_alignment (GTK_MISC (entry->progress_percentage), 1.0, 0.5);
+	gtk_box_pack_start (GTK_BOX (entry->box), entry->progress_percentage, FALSE, TRUE, 0);
 
 	entry->progress_bar = gtk_progress_bar_new ();
+	gtk_progress_bar_set_pulse_step (GTK_PROGRESS_BAR (entry->progress_bar), 0.02);
+	gtk_progress_bar_set_ellipsize (GTK_PROGRESS_BAR (entry->progress_bar), PANGO_ELLIPSIZE_END);
+	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (entry->progress_bar), 1.00);
 
-	gtk_box_pack_start (GTK_BOX (entry->box), entry->name, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (entry->box), entry->progress_bar, TRUE, TRUE, 0);
+        /* Get the font ascent for the current font and language */
+        context = gtk_widget_get_pango_context (entry->progress_bar);
+        fontdesc = pango_context_get_font_description (context);
+        lang = pango_context_get_language (context);
+        metrics = pango_context_get_metrics (context, fontdesc, lang);
+        ascent = pango_font_metrics_get_ascent (metrics) * 1.5 / PANGO_SCALE;
+        pango_font_metrics_unref (metrics);
+
+        /* Size our progress bar to be five ascents long */
+        gtk_widget_set_size_request (entry->progress_bar, ascent * 5, -1);
+
+	gtk_box_pack_start (GTK_BOX (entry->box), entry->progress_bar, FALSE, TRUE, 0);
 
 	gtk_size_group_add_widget (priv->size_group, entry->name);
 
@@ -493,6 +639,8 @@ miner_menu_entry_add (TrackerStatusIcon *icon,
 
 	if (!entry->active) {
 		gtk_widget_set_sensitive (entry->menu_item, FALSE);
+		gtk_widget_hide (entry->progress_bar);
+		gtk_widget_hide (entry->progress_percentage);
 	}
 
 	g_hash_table_replace (priv->miners, str, entry);
@@ -799,16 +947,32 @@ status_icon_set_status (TrackerStatusIcon *icon,
 			TrackerStatus      status)
 {
 	TrackerStatusIconPrivate *priv;
+	TrackerVisibility visibility;
 
 	priv = TRACKER_STATUS_ICON_GET_PRIVATE (icon);
+	visibility = tracker_icon_config_get_visibility (priv->config);
 
-	if (priv->current_status == status) {
+	if (priv->current_status == status &&
+	    priv->current_visibility == visibility) {
+		return;
+	}
+
+	priv->current_status = status;
+	priv->current_visibility = visibility;
+
+	if (visibility == TRACKER_SHOW_NEVER) {
+		gtk_menu_popdown (GTK_MENU (priv->miner_menu));
+		gtk_status_icon_set_visible (GTK_STATUS_ICON (icon), FALSE);
 		return;
 	}
 
 	switch (status) {
 	case STATUS_IDLE:
 		animate_indexing (icon, FALSE);
+
+		gtk_status_icon_set_visible (GTK_STATUS_ICON (icon),
+					     (visibility == TRACKER_SHOW_ALWAYS));
+
 		gtk_status_icon_set_from_pixbuf (GTK_STATUS_ICON (icon),
 						 priv->icons [ICON_IDLE]);
 		break;
@@ -818,14 +982,13 @@ status_icon_set_status (TrackerStatusIcon *icon,
 						 priv->icons [ICON_PAUSED]);
 		break;
 	case STATUS_INDEXING:
+		gtk_status_icon_set_visible (GTK_STATUS_ICON (icon), TRUE);
 		animate_indexing (icon, TRUE);
 		break;
 	default:
 		g_critical ("Unknown status '%d'", status);
 		g_assert_not_reached ();
 	}
-
-	priv->current_status = status;
 }
 
 GtkStatusIcon *
