@@ -38,6 +38,16 @@
 
 #include "tracker-miner-client.h"
 
+#define ABOUT                                                             \
+        "Tracker " PACKAGE_VERSION "\n"
+
+#define LICENSE                                                           \
+        "This program is free software and comes without any warranty.\n" \
+        "It is licensed under version 2 or later of the General Public "  \
+        "License which can be viewed at:\n"                               \
+        "\n"                                                              \
+        "  http://www.gnu.org/licenses/gpl.txt\n"
+
 static GMainLoop *main_loop;
 static GHashTable *miners_progress;
 static GHashTable *miners_status;
@@ -51,6 +61,7 @@ static gchar     *pause_reason;
 static gint       resume_cookie = -1;
 static gboolean   follow;
 static gboolean   detailed;
+static gboolean   print_version;
 
 static GOptionEntry entries[] = {
 	{ "follow", 'f', 0, G_OPTION_ARG_NONE, &follow,
@@ -89,6 +100,10 @@ static GOptionEntry entries[] = {
 	  N_("Resume a miner (you must use this with --miner)"),
 	  N_("COOKIE")
 	},
+	{ "version", 'V', 0, G_OPTION_ARG_NONE, &print_version,
+	  N_("Print version"),
+	  NULL 
+	},
 	{ NULL }
 };
 
@@ -285,7 +300,7 @@ manager_miner_paused_cb (TrackerMinerManager *manager,
 
 	miner_print_state (manager, miner_name,
 			   g_hash_table_lookup (miners_status, miner_name),
-			   g_value_get_double (gvalue), 
+			   gvalue ? g_value_get_double (gvalue) : 0.0, 
 			   TRUE, 
 			   TRUE);
 }
@@ -300,7 +315,7 @@ manager_miner_resumed_cb (TrackerMinerManager *manager,
 
 	miner_print_state (manager, miner_name,
 			   g_hash_table_lookup (miners_status, miner_name),
-			   g_value_get_double (gvalue), 
+			   gvalue ? g_value_get_double (gvalue) : 0.0, 
 			   TRUE, 
 			   FALSE);
 }
@@ -335,6 +350,13 @@ main (gint argc, gchar *argv[])
 	g_option_context_add_main_entries (context, entries, NULL);
 	g_option_context_parse (context, &argc, &argv, NULL);
 
+	if (print_version) {
+                g_print ("\n" ABOUT "\n" LICENSE "\n");
+		g_option_context_free (context);
+
+		return EXIT_SUCCESS;
+	}
+
 	if (pause_reason && resume_cookie != -1) {
 		gchar *help;
 
